#include <gear/graphics/renderer.h>
#include <gear/camera.h>
#include <gear/ui/gui.h>
#include <gear/ui/console.h>
#include <gear/resources.h>

#include <sokol/sokol_app.h>
#include <sokol/sokol_gfx.h>
#include <sokol/sokol_log.h>
#include <sokol/sokol_glue.h>

#include <log.c/log.h>

g_renderer* renderer;
g_camera* camera;
g_gui* gui;

void init() {
    sg_setup(&(sg_desc){
        .environment = sglue_environment(),
        .logger.func = slog_func,
    });
    log_info("Sokol graphics initialized successfully");

    renderer = g_renderer_create(G_CYAN);
    gui = g_gui_create();
    camera = g_camera_create(HMM_V3(0.0f, 0.0f, -3.0f), HMM_V3(0.0f, 0.0f, 0.0f), 70.0f);
    log_info("GUI Layers (%d):", gui->layers.size);
    for(sl_vec_it(layer, gui->layers)) {
        log_info("    %s", (*layer)->name);
    }
}

void frame() {
    g_gui_update(gui);

    g_renderer_begin(renderer);
    g_gui_render(sapp_width(), sapp_height());
    g_renderer_end(renderer);
}

void cleanup() {
    log_info("Shutting down");

    g_renderer_destroy(renderer);
    g_gui_destroy(gui);
    g_camera_destroy(camera);

    sg_shutdown();
}

void input(const sapp_event* event) {
    snk_handle_event(event);
}

sapp_desc sokol_main(int argc, char* argv[]) {
    (void)argc; (void)argv;
    FILE* log_fd = fopen("log.txt", "w");
    log_add_fp(log_fd, LOG_TRACE);

    log_info("Command line arguments (%d):", argc);
    for(i32 i = 0; i < argc; i++) {
        log_info("    %s", argv[i]);
    }

    return (sapp_desc){
        .init_cb = init,
        .frame_cb = frame,
        .cleanup_cb = cleanup,
        .event_cb = input,
        .width = 640,
        .height = 480,
        .window_title = "Gear",
        .icon.sokol_default = true,
        .logger.func = slog_func,
        .swap_interval = 0
    };
}
